#!/bin/bash -e
# This script builds a vserver and performs the minimal guest
# initialization.
#  Parameters:
# * BuildVserverGuestConfigName: Name of the vserver, used to to store the
#   configuration in /etc/vservers
# * BuildVserverGuestTemplateFile: Template file to use during building
# * BuildVserverGuestHostname: The hostname of the guest
# * BuildVserverGuestHostnameAliases: Array with hostname alias
# * BuildVserverGuestFqdn:
# * BuildVserverGuestContextId: Context ID of new vserver, assigned
#   by vserver utils if missing
# * BuildVserverGuestNetworkFlags: If set, value is written to nflags
# * BuildVserverGuestNetworkIf[n]Ip: IP/mask to use for interface [n]
# * BuildVserverGuestNetworkIf[n]Name:
# * BuildVserverGuestLocalhostIp: IP of localhost inside guest. If
#   empty, 127.0.0.1 is used. Old guests or guest with internal
#   loopback disabled might need this value to be set, e.g. to
#   ${BuildVserverGuestNetworkIf0Ip%/*}
# * BuildVserverGuestInitStyle: If defined, value is stored to vserver
#   configuration apps/init/style. Options: plain, sysv (default),
#   gentoo, minit
# * BuildVserverGuestUpstartEnable: If set to true, this script
#   will modify guest to make upstart work.
#
# Resources:
# * Data/BuildVserverGuest/HostsFileAddons.txt: These entries are added
#   to the minimal /etc/hosts file during building. Use this addon if
#   the entries are needed for the first "vserver start" at the end of
#   the script, in most cases the /etc/hosts is replaced from within
#   guest when continuing setup there.
# * Data/BuildVserverGuest/Root: These files are copied to root
#   of new vserver before starting it

. ./Config.sh

_setup_buildCmd=(vserver - build --hostname "${BuildVserverGuestHostname}" -m template -n "${BuildVserverGuestConfigName}")
if [ "${BuildVserverGuestContextId}" != "" ]; then
  _setup_buildCmd+=(--context "${BuildVserverGuestContextId}")
fi
_setup_buildCmd+=(-- -t "${BuildVserverGuestTemplateFile}")
"${_setup_buildCmd[@]}"

_setup_vsConfDir="/etc/vservers/${BuildVserverGuestConfigName}"

_setup_interfaceNum=0
while true; do
  _setup_ipDataVarName="BuildVserverGuestNetworkIf${_setup_interfaceNum}Ip"
  _setup_ipAddress="${!_setup_ipDataVarName}"
  if [ "${_setup_ipAddress}" = "" ]; then break; fi
  mkdir -p -- "${_setup_vsConfDir}/interfaces/${_setup_interfaceNum}"

  _setup_ipDataVarName="BuildVserverGuestNetworkIf${_setup_interfaceNum}Name"
  echo "${!_setup_ipDataVarName}" > "${_setup_vsConfDir}/interfaces/${_setup_interfaceNum}/dev"
  echo "${_setup_ipAddress}" > "${_setup_vsConfDir}/interfaces/${_setup_interfaceNum}/ip"
  let _setup_interfaceNum=_setup_interfaceNum+1
done

if [ "${BuildVserverGuestNetworkFlags}" != "" ]; then
  echo "${BuildVserverGuestNetworkFlags}" > "${_setup_vsConfDir}/nflags"
fi

if [ "${BuildVserverGuestInitStyle}" != "" ]; then
  echo "${BuildVserverGuestInitStyle}" > "${_setup_vsConfDir}/apps/init/style"
fi

echo "default" > "${_setup_vsConfDir}/apps/init/mark"

# Set the guest hostname/mailname before starting
echo "${BuildVserverGuestHostname}" > "${_setup_vsConfDir}/vdir/etc/hostname"
echo "${BuildVserverGuestFqdn}" > "${_setup_vsConfDir}/vdir/etc/mailname"

# Build a primitive hosts file for starting, might be replaced
# during guest-internal initialization. Use the same code from
# /Common/SetupHostBasePreUpstart.sh

if [ "${BuildVserverGuestLocalhostIp}" = "" ]; then
  BuildVserverGuestLocalhostIp=127.0.0.1
fi
_setup_hostsFile="${_setup_vsConfDir}/vdir/etc/hosts"
cat <<EOF > "${_setup_hostsFile}"
# Minimal host file created by setup process
${BuildVserverGuestLocalhostIp}	localhost
EOF

_setup_hostNames=()
if [ "${BuildVserverGuestFqdn}" != "" ] && [ "${BuildVserverGuestFqdn}" != "${BuildVserverGuestHostname}" ]; then
# Put the fqdn first, if it exists, otherwise "hostname -f"
# will not return the fqdn, exim depends on that
  _setup_hostNames+=("${BuildVserverGuestFqdn}")
fi
if [ "${BuildVserverGuestHostname}" != "localhost" ]; then
  _setup_hostNames+=("${BuildVserverGuestHostname}")
fi
if [ "${#BuildVserverGuestHostnameAliases[@]}" != "0" ]; then
  _setup_hostNames+=("${BuildVserverGuestHostnameAliases[@]}")
fi

if [ "${#_setup_hostNames[@]}" != "0" ]; then
  echo "${BuildVserverGuestLocalhostIp}	${_setup_hostNames[@]}" >> "${_setup_hostsFile}"
fi
echo "" >> "${_setup_hostsFile}"

# Append the host file addons if available
if test -e "Data/BuildVserverGuest/HostsFileAddons.txt"; then
  cat Data/BuildVserverGuest/HostsFileAddons.txt >> "${_setup_hostsFile}"
fi

# Copy root disk data if available
if test -d "Data/BuildVserverGuest/Root"; then
  cp -aT -- "Data/BuildVserverGuest/Root" "${_setup_vsConfDir}/vdir/"
fi

# BUGFIX: ubuntu-precise guests cannot start because of directories
# missing in template. It is not clear, if this is a bug in debootstrap,
# util-vserver or setup process. So create those if missing. This
# problem was observed with ubuntu-precise beta1 for the first time.
# FIXME: Check if ubuntu release has same problem
for _setup_checkDirName in /var/lock /var/run; do
  if test -L "${_setup_vsConfDir}/vdir/${_setup_checkDirName}"; then
    rm -r -- "${_setup_vsConfDir}/vdir/${_setup_checkDirName}"
    mkdir -p -- "${_setup_vsConfDir}/vdir/${_setup_checkDirName}"
  elif ! test -e "${_setup_vsConfDir}/vdir/${_setup_checkDirName}"; then
    mkdir -p -- "${_setup_vsConfDir}/vdir/${_setup_checkDirName}"
  fi
done

# klogd will run into timeout when starting, blocking installation
# procedure for some minutes. Fix it by disabling klogd. After
# first startup, klogd should be replaced by sysklogd.
if test -f "${_setup_vsConfDir}/vdir/etc/default/klogd"; then
  echo "exit 0" >> "${_setup_vsConfDir}/vdir/etc/default/klogd"
fi

if [ "${BuildVserverGuestUpstartEnable}" = "true" ]; then
  cat <<EOF > "${_setup_vsConfDir}/vdir/etc/init/vserver-integration.conf"
# Trigger events to make upstart in vserver guest work. See
# http://linux-vserver.org/Upstart_issues for explanations.
# Setup/Common/Vserver/BuildVserverGuest.sh V20121120
start on startup
script
  touch /var/run/utmp
  chown root:utmp /var/run/utmp
  chmod 664 /var/run/utmp
  initctl emit virtual-filesystems
  initctl emit local-filesystems
  initctl emit remote-filesystems
# Unclear why this may hang sometimes
  initctl emit --no-wait filesystem
  initctl emit net-device-up
  initctl emit static-network-up
end script
EOF

# Remove some problematic init scripts.
# DO NOT REMOVE control-alt-delete.conf! It seems to interact with normal
# guest shutdown process!
  (
    cd "${_setup_vsConfDir}/vdir/etc/init"
    rm -- console-setup.conf dmesg.conf hostname.conf hwclock* mountall* networking.conf network-interface.conf procps.conf tty* udev* upstart-udev-bridge.conf
    sed -i -r -e 's/^(console output)$/# \1/' *.conf
  )

# 2013-02-07 RFi: Copy fake /dev/console to guest, otherwise upstart
# will complain even when console output is not enabled.
  cp -a /dev/null "${_setup_vsConfDir}/vdir/dev/console"
fi

# Reload the firewall, it might create additional rules when a new
# vserver guest is present
if test -x /etc/init.d/ait-iptablesfirewall; then
  /etc/init.d/ait-iptablesfirewall start
fi

# Start the guest
vserver "${BuildVserverGuestConfigName}" start
